@@ -22,6 +22,7 @@ from glue.utils.qt import get_qapp
 from glue.app.qt.actions import action
 from glue.dialogs.data_wizard.qt import data_wizard
 from glue.dialogs.link_editor.qt import LinkEditor
+from glue.dialogs.component_arithmetic.qt import ArithmeticEditorWidget
 from glue.app.qt.edit_subset_mode_toolbar import EditSubsetModeToolBar
 from glue.app.qt.mdi_area import GlueMdiArea
 from glue.app.qt.layer_tree_widget import PlotAction, LayerTreeWidget
@@ -133,9 +134,13 @@ class GlueLogger(QtWidgets.QWidget):
 
     def _set_console_button(self, attention):
         if attention:
-            self.button_console.setStyleSheet('color: red; text-decoration: underline;')
+            style = 'color: red; text-decoration: underline;'
         else:
-            self.button_console.setStyleSheet(self.button_stylesheet)
+            style = self.button_stylesheet
+        try:
+            self.button_console.setStyleSheet(style)
+        except RuntimeError:  # Prevent RuntimeError: wrapped C/C++ object of type QToolButton has been deleted
+            pass
 
     def write(self, message):
         """
@@ -219,6 +224,34 @@ class ExportHelper(object):
             return saver(app, label)
 
 
+class ImportHelper(object):
+
+    def __init__(self, app):
+        self.app = weakref.ref(app)
+
+    def _choose_load_data_wizard(self, *args):
+        self._choose_load_data(data_importer=data_wizard)
+
+    def _choose_load_data(self, data_importer=None):
+
+        app = self.app()
+        if app is None:
+            return
+
+        if data_importer is None:
+            app.add_datasets(app.data_collection, data_wizard())
+        else:
+            data = data_importer()
+            if not isinstance(data, list):
+                raise TypeError("Data loader should return list of "
+                                "Data objects")
+            for item in data:
+                if not isinstance(item, Data):
+                    raise TypeError("Data loader should return list of "
+                                    "Data objects")
+            app.add_datasets(app.data_collection, data)
+
+
 class GlueApplication(Application, QtWidgets.QMainWindow):
 
     """ The main GUI application for the Qt frontend"""
@@ -234,6 +267,7 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
             self._original_icon = self._original_app.windowIcon()
 
         self._export_helper = ExportHelper(self)
+        self._import_helper = ImportHelper(self)
 
         # Now we can get the application instance, which involves setting it
         # up if it doesn't already exist.
@@ -342,7 +376,7 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
         self._button_open_data.setText("Open Data")
         self._button_open_data.setIcon(get_icon('glue_open'))
         self._button_open_data.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
-        self._button_open_data.clicked.connect(self._choose_load_data_wizard)
+        self._button_open_data.clicked.connect(self._import_helper._choose_load_data_wizard)
 
         self._data_toolbar.addWidget(self._button_open_data)
 
@@ -359,7 +393,6 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
         self._button_link_data.setIcon(get_icon('glue_link'))
         self._button_link_data.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
         self._button_link_data.clicked.connect(self._set_up_links)
-        self._on_data_collection_change()
 
         self._data_toolbar.addWidget(self._button_link_data)
 
@@ -392,7 +425,7 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
         self._button_edit_components.setText("Add/edit arithmetic attributes")
         self._button_edit_components.setIcon(get_icon('pencil'))
         self._button_edit_components.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
-        self._button_edit_components.clicked.connect(nonpartial(self._layer_widget._create_component))
+        self._button_edit_components.clicked.connect(self._artihmetic_dialog)
 
         self._data_toolbar.addWidget(self._button_edit_components)
 
@@ -405,6 +438,8 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
 
         self.addToolBar(self._data_toolbar)
 
+        self._on_data_collection_change()
+
         # Selection mode toolbar
 
         tbar = EditSubsetModeToolBar(parent=self)
@@ -447,8 +482,14 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
 
         self._hub.subscribe(self, DataCollectionMessage, handler=self._on_data_collection_change)
 
+    def _artihmetic_dialog(self, *event):
+        dialog = ArithmeticEditorWidget(self.data_collection)
+        dialog.exec_()
+
     def _on_data_collection_change(self, *event):
+        self._button_save_data.setEnabled(len(self.data_collection) > 0)
         self._button_link_data.setEnabled(len(self.data_collection) > 1)
+        self._button_edit_components.setEnabled(len(self.data_collection) > 0)
 
     def keyPressEvent(self, event):
         if self.current_tab.activeSubWindow() and self.current_tab.activeSubWindow().widget():
@@ -774,23 +815,6 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
         window.show()
         window.exec_()
 
-    def _choose_load_data_wizard(self, *args):
-        self._choose_load_data(data_importer=data_wizard)
-
-    def _choose_load_data(self, data_importer=None):
-        if data_importer is None:
-            self.add_datasets(self.data_collection, data_wizard())
-        else:
-            data = data_importer()
-            if not isinstance(data, list):
-                raise TypeError("Data loader should return list of "
-                                "Data objects")
-            for item in data:
-                if not isinstance(item, Data):
-                    raise TypeError("Data loader should return list of "
-                                    "Data objects")
-            self.add_datasets(self.data_collection, data)
-
     def _choose_save_data(self, *args):
         dialog = SaveDataDialog(data_collection=self.data_collection, parent=self)
         dialog.exec_()
@@ -843,7 +867,7 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
         # also add it again below in the Import menu for consistency.
         a = action("&Open Data Set", self, tip="Open a new data set",
                    shortcut=QtGui.QKeySequence.Open)
-        a.triggered.connect(self._choose_load_data_wizard)
+        a.triggered.connect(self._import_helper._choose_load_data_wizard)
         self._actions['data_new'] = a
 
         # We now populate the "Import data" menu
@@ -853,13 +877,12 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
 
         # Add default file loader (later we can add this to the registry)
         a = action("Import from file", self, tip="Import from file")
-        a.triggered.connect(self._choose_load_data_wizard)
+        a.triggered.connect(self._import_helper._choose_load_data_wizard)
         acts.append(a)
 
-        for i in importer:
-            label, data_importer = i
+        for label, data_importer in importer:
             a = action(label, self, tip=label)
-            a.triggered.connect(self._choose_load_data_wizard)
+            a.triggered.connect(nonpartial(self._import_helper._choose_load_data, data_importer))
             acts.append(a)
 
         self._actions['data_importers'] = acts
@@ -1056,6 +1079,11 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
         ga = GlueApplication()
         ga.start(block=False)
 
+        # NOTE: we need to keep a reference to this new application object
+        # otherwise it will immediately garbage collect - this is a hack and
+        # we should find a better solution in future.
+        self._new_application = ga
+
         # We need to close this after we open the next application otherwise
         # Qt will quit since there are no actively open windows.
         self.close()
@@ -1214,6 +1242,12 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
         with set_cursor_cm(Qt.WaitCursor):
             app = self.restore_session(path)
             app.setGeometry(self.geometry())
+
+            # NOTE: we need to keep a reference to this new application object
+            # otherwise it will immediately garbage collect - this is a hack and
+            # we should find a better solution in future.
+            self._new_application = app
+
             self.close()
 
     def closeEvent(self, event):
