@@ -102,6 +102,21 @@ class ComboHelper(HubListener):
         prop = getattr(type(self.state), self.selection_property)
         return prop.set_display_func(self.state, display)
 
+    def _on_rename(self, msg):
+        # If a component ID is renamed, we don't need to refresh because the
+        # list of actual component IDs is the same as before. However, we do
+        # need to trigger a refresh of any GUI combos that use this, so we
+        # make the property notify a change. However, if we are inside a
+        # delay_callback block, the property will not be enabled, and notify()
+        # won't have any effect, in which case we set the 'force_next_sync'
+        # option which means that when exiting from the delay_callback block,
+        # this property will show up as having changed
+        prop = getattr(type(self.state), self.selection_property)
+        if prop.enabled(self.state):
+            prop.notify(self.state, self.selection, self.selection)
+        else:
+            prop.force_next_sync(self.state)
+
 
 class ComponentIDComboHelper(ComboHelper):
     """
@@ -360,34 +375,19 @@ class ComponentIDComboHelper(ComboHelper):
 
         self.choices = choices
 
-    def _on_rename(self, msg):
-        # If a component ID is renamed, we don't need to refresh because the
-        # list of actual component IDs is the same as before. However, we do
-        # need to trigger a refresh of any GUI combos that use this, so we
-        # make the property notify a change. However, if we are inside a
-        # delay_callback block, the property will not be enabled, and notify()
-        # won't have any effect, in which case we set the 'force_next_sync'
-        # option which means that when exiting from the delay_callback block,
-        # this property will show up as having changed
-        prop = getattr(type(self.state), self.selection_property)
-        if prop.enabled(self.state):
-            prop.notify(self.state, self.selection, self.selection)
-        else:
-            prop.force_next_sync(self.state)
-
     def _filter_msg(self, msg):
-        return msg.data in self._data or msg.sender in self._data_collection
+        return msg.sender in self._data
 
     def register_to_hub(self, hub):
         hub.subscribe(self, DataRenameComponentMessage,
                       handler=self._on_rename,
-                      filter=lambda msg: msg.sender in self._data)
+                      filter=self._filter_msg)
         hub.subscribe(self, DataReorderComponentMessage,
                       handler=self.refresh,
-                      filter=lambda msg: msg.sender in self._data)
+                      filter=self._filter_msg)
         hub.subscribe(self, ComponentsChangedMessage,
                       handler=self.refresh,
-                      filter=lambda msg: msg.sender in self._data)
+                      filter=self._filter_msg)
         if self._data_collection is not None:
             hub.subscribe(self, DataCollectionDeleteMessage,
                           handler=self._remove_data)
@@ -469,6 +469,12 @@ class BaseDataComboHelper(ComboHelper):
     def register_to_hub(self, hub):
         pass
 
+    def _on_data_update(self, msg):
+        if msg.attribute == 'label':
+            self._on_rename(msg)
+        else:
+            self.refresh()
+
 
 class ManualDataComboHelper(BaseDataComboHelper):
     """
@@ -513,14 +519,17 @@ class ManualDataComboHelper(BaseDataComboHelper):
         except AttributeError:  # PY2
             self._datasets[:] = []
         for data in unique_data_iter(datasets):
-            self._datasets.append(data)
+            self.append_data(data, refresh=False)
         self.refresh()
 
-    def append_data(self, data):
+    def append_data(self, data, refresh=True):
         if data in self._datasets:
             return
+        if self.hub is None and data.hub is not None:
+            self.hub = data.hub
         self._datasets.append(data)
-        self.refresh()
+        if refresh:
+            self.refresh()
 
     def remove_data(self, data):
         if data not in self._datasets:
@@ -528,16 +537,25 @@ class ManualDataComboHelper(BaseDataComboHelper):
         self._datasets.remove(data)
         self.refresh()
 
+    def _remove_data_msg(self, msg):
+        self.remove_data(msg.data)
+
+    def _filter_msg(self, msg):
+        return msg.sender in self._datasets
+
+    def _filter_msg_dc(self, msg):
+        return msg.sender is self._data_collection
+
     def register_to_hub(self, hub):
 
         super(ManualDataComboHelper, self).register_to_hub(hub)
 
         hub.subscribe(self, DataUpdateMessage,
-                      handler=self.refresh,
-                      filter=lambda msg: msg.sender in self._datasets)
+                      handler=self._on_data_update,
+                      filter=self._filter_msg)
         hub.subscribe(self, DataCollectionDeleteMessage,
-                      handler=lambda msg: self.remove_data(msg.data),
-                      filter=lambda msg: msg.sender is self._data_collection)
+                      handler=self._remove_data_msg,
+                      filter=self._filter_msg_dc)
 
 
 class DataCollectionComboHelper(BaseDataComboHelper):
@@ -564,14 +582,20 @@ class DataCollectionComboHelper(BaseDataComboHelper):
 
         self.refresh()
 
+    def _filter_msg_in(self, msg):
+        return msg.sender in self._datasets
+
+    def _filter_msg_is(self, msg):
+        return msg.sender is self._datasets
+
     def register_to_hub(self, hub):
         super(DataCollectionComboHelper, self).register_to_hub(hub)
         hub.subscribe(self, DataUpdateMessage,
-                      handler=self.refresh,
-                      filter=lambda msg: msg.sender in self._datasets)
+                      handler=self._on_data_update,
+                      filter=self._filter_msg_in)
         hub.subscribe(self, DataCollectionAddMessage,
                       handler=self.refresh,
-                      filter=lambda msg: msg.sender is self._datasets)
+                      filter=self._filter_msg_is)
         hub.subscribe(self, DataCollectionDeleteMessage,
                       handler=self.refresh,
-                      filter=lambda msg: msg.sender is self._datasets)
+                      filter=self._filter_msg_is)
