@@ -7,7 +7,7 @@ from matplotlib.patches import Ellipse, Polygon, Rectangle, Path as MplPath, Pat
 from matplotlib.transforms import IdentityTransform, blended_transform_factory
 
 from glue.core.exceptions import UndefinedROI
-from glue.utils import points_inside_poly
+from glue.utils import points_inside_poly, iterate_chunks
 
 
 np.seterr(all='ignore')
@@ -74,6 +74,31 @@ class Roi(object):  # pragma: no cover
         """
         raise NotImplementedError()
 
+    def contains3d(self, x, y, z):
+        """Return true/false for each x/y/z pair.
+
+        Parameters
+        ----------
+        x : :class:`numpy.ndarray`
+            Array of x locations
+        y : :class:`numpy.ndarray`
+            Array of y locations
+        z : :class:`numpy.ndarray`
+            Array of z locations
+
+        Returns
+        -------
+        :class:`numpy.ndarray`
+            A boolean array, where each element is `True` if the corresponding
+            (x,y,z) tuple is inside the Roi.
+
+        Raises
+        ------
+        UndefinedROI
+            if not defined
+        """
+        raise NotImplementedError()
+
     def center(self):
         """Return the (x,y) coordinates of the ROI center"""
         raise NotImplementedError()
@@ -89,7 +114,7 @@ class Roi(object):  # pragma: no cover
     def to_polygon(self):
         """ Returns a tuple of x and y points, approximating the ROI
         as a polygon."""
-        raise NotImplementedError
+        raise NotImplementedError()
 
     def copy(self):
         """
@@ -97,6 +122,12 @@ class Roi(object):  # pragma: no cover
         """
         return copy.copy(self)
 
+    def transformed(self, xfunc=None, yfunc=None):
+        """
+        A transformed version of the ROI
+        """
+        raise NotImplementedError()
+
 
 class PointROI(Roi):
 
@@ -190,6 +221,7 @@ class RectangularROI(Roi):
             A list of True/False values, for whether each (x,y)
             point falls within the ROI
         """
+
         if not self.defined():
             raise UndefinedROI
 
@@ -219,13 +251,23 @@ class RectangularROI(Roi):
 
     def to_polygon(self):
         if self.defined():
-            return [self.xmin, self.xmax, self.xmax, self.xmin, self.xmin], \
-                [self.ymin, self.ymin, self.ymax, self.ymax, self.ymin]
+            return (np.array([self.xmin, self.xmax, self.xmax, self.xmin, self.xmin]),
+                    np.array([self.ymin, self.ymin, self.ymax, self.ymax, self.ymin]))
         else:
             return [], []
 
+    def transformed(self, xfunc=None, yfunc=None):
+        xmin = self.xmin if xfunc is None else xfunc(self.xmin)
+        xmax = self.xmax if xfunc is None else xfunc(self.xmax)
+        ymin = self.ymin if yfunc is None else yfunc(self.ymin)
+        ymax = self.ymax if yfunc is None else yfunc(self.ymax)
+        return RectangularROI(xmin=xmin, xmax=xmax, ymin=ymin, ymax=ymax)
+
     def __gluestate__(self, context):
-        return dict(xmin=self.xmin, xmax=self.xmax, ymin=self.ymin, ymax=self.ymax)
+        return dict(xmin=context.do(self.xmin),
+                    xmax=context.do(self.xmax),
+                    ymin=context.do(self.ymin),
+                    ymax=context.do(self.ymax))
 
     @classmethod
     def __setgluestate__(cls, rec, context):
@@ -282,6 +324,15 @@ class RangeROI(Roi):
         coord = x if self.ori == 'x' else y
         return (coord > self.min) & (coord < self.max)
 
+    def transformed(self, xfunc=None, yfunc=None):
+        if self.ori == 'x':
+            vmin = self.min if xfunc is None else xfunc(self.min)
+            vmax = self.max if xfunc is None else xfunc(self.max)
+        else:
+            vmin = self.min if yfunc is None else yfunc(self.min)
+            vmax = self.max if yfunc is None else yfunc(self.max)
+        return RangeROI(orientation=self.ori, min=vmin, max=vmax)
+
     def reset(self):
         self.min = None
         self.max = None
@@ -299,7 +350,7 @@ class RangeROI(Roi):
             return [], []
 
     def __gluestate__(self, context):
-        return dict(ori=self.ori, min=self.min, max=self.max)
+        return dict(ori=self.ori, min=context.do(self.min), max=context.do(self.max))
 
     @classmethod
     def __setgluestate__(cls, rec, context):
@@ -394,8 +445,13 @@ class CircularROI(Roi):
         y = self.yc + self.radius * np.sin(theta)
         return x, y
 
+    def transformed(self, xfunc=None, yfunc=None):
+        return PolygonalROI(*self.to_polygon()).transformed(xfunc=xfunc, yfunc=yfunc)
+
     def __gluestate__(self, context):
-        return dict(xc=self.xc, yc=self.yc, radius=self.radius)
+        return dict(xc=context.do(self.xc),
+                    yc=context.do(self.yc),
+                    radius=context.do(self.radius))
 
     @classmethod
     def __setgluestate__(cls, rec, context):
@@ -419,6 +475,11 @@ class VertexROIBase(Roi):
         if self.vy is None:
             self.vy = []
 
+    def transformed(self, xfunc=None, yfunc=None):
+        vx = self.vx if xfunc is None else xfunc(np.asarray(self.vx))
+        vy = self.vy if yfunc is None else yfunc(np.asarray(self.vy))
+        return self.__class__(vx=vx, vy=vy)
+
     def add_point(self, x, y):
         """
         Add another vertex to the ROI
@@ -474,12 +535,13 @@ class VertexROIBase(Roi):
         return self.vx, self.vy
 
     def __gluestate__(self, context):
-        return dict(vx=np.asarray(self.vx).tolist(),
-                    vy=np.asarray(self.vy).tolist())
+        return dict(vx=context.do(np.asarray(self.vx)),
+                    vy=context.do(np.asarray(self.vy)))
 
     @classmethod
     def __setgluestate__(cls, rec, context):
-        return cls(vx=rec['vx'], vy=rec['vy'])
+        return cls(vx=context.object(rec['vx']),
+                   vy=context.object(rec['vy']))
 
 
 class PolygonalROI(VertexROIBase):
@@ -524,6 +586,85 @@ class PolygonalROI(VertexROIBase):
         self.vy = list(map(lambda y: y + ydelta, self.vy))
 
 
+class Projected3dROI(Roi):
+    """"A region of interest defined in screen coordinates.
+
+    The screen coordinates are defined by the projection matrix.
+    The projection matrix converts homogeneous coordinates (x, y, z, w), where
+    w is implicitly 1, to homogeneous screen coordinates (usually the product
+    of the world and projection matrix).
+    """
+
+    def __init__(self, roi_2d=None, projection_matrix=None):
+        super(Projected3dROI, self).__init__()
+        self.roi_2d = roi_2d
+        self.projection_matrix = np.asarray(projection_matrix)
+
+    def contains3d(self, x, y, z):
+        """
+        Test whether the projected coordinates are contained in the 2d ROI.
+        """
+
+        if not self.defined():
+            raise UndefinedROI
+
+        x = np.asarray(x)
+        y = np.asarray(y)
+        z = np.asarray(z)
+
+        # Since the projection can significantly increase the memory usage, we
+        # do the following operation in chunks. In future we could likely use
+        # e.g. vaex, dask, or other multi-threaded/fast libraries to speed this
+        # and other ROI code up.
+
+        mask = np.zeros(x.shape, dtype=bool)
+
+        for slices in iterate_chunks(x.shape, n_max=1000000):
+
+            # Work in homogeneous coordinates so we can support perspective
+            # projections as well
+            x_sub, y_sub, z_sub = x[slices], y[slices], z[slices]
+            vertices = np.array([x_sub, y_sub, z_sub, np.ones(x_sub.shape)])
+
+            # The following returns homogeneous screen coordinates
+            screen_h = np.tensordot(self.projection_matrix,
+                                    vertices, axes=(1, 0))
+
+            # Convert to screen coordinates, as we don't care about z
+            screen_x, screen_y = screen_h[:2] / screen_h[3]
+
+            mask[slices] = self.roi_2d.contains(screen_x, screen_y)
+
+        return mask
+
+    def __gluestate__(self, context):
+        return dict(roi_2d=context.id(self.roi_2d), projection_matrix=self.projection_matrix.tolist())
+
+    @classmethod
+    def __setgluestate__(cls, rec, context):
+        return cls(roi_2d=context.object(rec['roi_2d']), projection_matrix=np.asarray(rec['projection_matrix']))
+
+    # TODO: these methods forward directly to roi_2d, not sure if this makes sense for all
+
+    def contains(self, x, y):
+        return self.roi_2d.contains(x, y)
+
+    def center(self):
+        return self.roi_2d.center()
+
+    def move_to(self, x, y):
+        return self.roi_2d.move_to(x, y)
+
+    def defined(self):
+        return self.roi_2d.defined()
+
+    def to_polygon(self):
+        return self.roi_2d.to_polygon()
+
+    def transformed(self, xfunc=None, yfunc=None):
+        return self.roi_2d.transformed(xfunc, yfunc)
+
+
 class Path(VertexROIBase):
 
     def __str__(self):
@@ -540,13 +681,13 @@ class AbstractMplRoi(object):  # pragma: no cover
     Matplotlib user events to edit/display ROIs
     """
 
-    def __init__(self, axes):
+    def __init__(self, axes, roi=None):
         """
         :param axes: The Matplotlib Axes object to draw to
         """
 
         self._axes = axes
-        self._roi = self._roi_factory()
+        self._roi = roi or self._roi_factory()
         self._previous_roi = None
         self._mid_selection = False
         self._scrubbing = False
@@ -1046,11 +1187,11 @@ class MplPolygonalROI(AbstractMplRoi):
                    the visual properties of the ROI
     """
 
-    def __init__(self, axes):
+    def __init__(self, axes, roi=None):
         """
         :param axes: A matplotlib Axes object to attach the graphical ROI to
         """
-        AbstractMplRoi.__init__(self, axes)
+        AbstractMplRoi.__init__(self, axes, roi=roi)
         self.plot_opts = {'edgecolor': PATCH_COLOR, 'facecolor': PATCH_COLOR,
                           'alpha': 0.3}
 
@@ -1083,12 +1224,12 @@ class MplPolygonalROI(AbstractMplRoi):
         # Refresh
         self._axes.figure.canvas.draw()
 
-    def start_selection(self, event):
+    def start_selection(self, event, scrubbing=False):
 
         if event.inaxes != self._axes:
             return False
 
-        if event.key == SCRUBBING_KEY:
+        if scrubbing or event.key == SCRUBBING_KEY:
             if not self._roi.defined():
                 return False
             elif not self._roi.contains(event.xdata, event.ydata):
@@ -1096,7 +1237,7 @@ class MplPolygonalROI(AbstractMplRoi):
 
         self._roi_store()
 
-        if event.key == SCRUBBING_KEY:
+        if scrubbing or event.key == SCRUBBING_KEY:
             self._scrubbing = True
             self._cx = event.xdata
             self._cy = event.ydata
