@@ -5,7 +5,6 @@ from collections import defaultdict
 import numpy as np
 
 from glue.core import Subset
-from glue.utils import nanmin, nanmax
 from glue.external.echo import (delay_callback, CallbackProperty,
                                 HasCallbackProperties, CallbackList)
 from glue.core.state import saver, loader
@@ -315,8 +314,8 @@ class StateAttributeLimitsHelper(StateAttributeCacheHelper):
             percentile = 100
             log = False
         else:
-            percentile = self.percentile or 100
-            log = self.log or False
+            percentile = getattr(self, 'percentile', None) or 100
+            log = getattr(self, 'log', None) or False
 
         if not force and (percentile == 'Custom' or not hasattr(self, 'data') or self.data is None):
 
@@ -353,32 +352,39 @@ class StateAttributeLimitsHelper(StateAttributeCacheHelper):
             if data_values.dtype.kind != 'M':
                 data_values = data_values[np.isfinite(data_values)]
 
-            if percentile == 100:
+            if data_values.size > 0:
+
+                if percentile == 100:
+
+                    if data_values.dtype.kind == 'M':
+                        lower = data_values.min()
+                        upper = data_values.max()
+                    else:
+                        lower = np.min(data_values)
+                        upper = np.max(data_values)
 
-                if data_values.dtype.kind == 'M':
-                    lower = data_values.min()
-                    upper = data_values.max()
                 else:
-                    lower = np.min(data_values)
-                    upper = np.max(data_values)
+
+                    lower = np.percentile(data_values, exclude)
+                    upper = np.percentile(data_values, 100 - exclude)
+
+                if self.data_component.categorical:
+                    lower = np.floor(lower - 0.5) + 0.5
+                    upper = np.ceil(upper + 0.5) - 0.5
+
+                if log:
+                    value_range = np.log10(upper / lower)
+                    lower /= 10.**(value_range * self.margin)
+                    upper *= 10.**(value_range * self.margin)
+                else:
+                    value_range = upper - lower
+                    lower -= value_range * self.margin
+                    upper += value_range * self.margin
 
             else:
 
-                lower = np.percentile(data_values, exclude)
-                upper = np.percentile(data_values, 100 - exclude)
-
-            if self.data_component.categorical:
-                lower = np.floor(lower - 0.5) + 0.5
-                upper = np.ceil(upper + 0.5) - 0.5
-
-            if log:
-                value_range = np.log10(upper / lower)
-                lower /= 10.**(value_range * self.margin)
-                upper *= 10.**(value_range * self.margin)
-            else:
-                value_range = upper - lower
-                lower -= value_range * self.margin
-                upper += value_range * self.margin
+                lower = 0.
+                upper = 1.
 
             self.set(lower=lower, upper=upper, percentile=percentile, log=log)
 
@@ -482,12 +488,17 @@ class StateAttributeHistogramHelper(StateAttributeCacheHelper):
 
                 values = self.data_values
 
-                if comp.datetime:
+                # NOTE: we can't use np.nanmin/np.nanmax or nanpercentile below as
+                # they don't exclude inf/-inf
+                if values.dtype.kind != 'M':
+                    values = values[np.isfinite(values)]
+
+                if values.size > 0:
                     lower = values.min()
                     upper = values.max()
                 else:
-                    lower = nanmin(values)
-                    upper = nanmax(values)
+                    lower = 0.
+                    upper = 1.
 
             self.set(lower=lower, upper=upper, n_bin=n_bin)
 
