@@ -1,11 +1,17 @@
+from __future__ import absolute_import, division, print_function
+
 import numpy as np
+
 from numpy.testing import assert_allclose
 
 from glue.external.echo import CallbackProperty, ListCallbackProperty
 from glue.core import Data, DataCollection
+from glue.utils import nanmedian
 
 from .test_state import clone
-from ..state_objects import State, StateAttributeLimitsHelper, StateAttributeSingleValueHelper, StateAttributeHistogramHelper
+from ..state_objects import (State, StateAttributeLimitsHelper,
+                             StateAttributeSingleValueHelper,
+                             StateAttributeHistogramHelper)
 
 
 class SimpleTestState(State):
@@ -166,7 +172,7 @@ class TestStateAttributeSingleValueHelper():
         self.state = SimpleState()
 
         self.helper = StateAttributeSingleValueHelper(self.state, attribute='comp',
-                                                      function=np.nanmedian, value='val')
+                                                      function=nanmedian, value='val')
 
         self.state.data = self.data
 
@@ -369,7 +375,6 @@ def test_histogram_helper_common_n_bin_active():
     assert state.n_bin == 11
 
 
-
 def test_limits_helper_initial_values():
 
     # Regression test for a bug that occurred if the limits cache was empty
@@ -396,3 +401,83 @@ def test_limits_helper_initial_values():
 
     assert helper.lower == 1
     assert helper.upper == 2
+
+
+class DatetimeState(State):
+    a = CallbackProperty()
+
+
+def test_state_serialization_datetime64():
+
+    state1 = DatetimeState()
+    state1.a = np.datetime64(100, 'D')
+
+    state2 = clone(state1)
+
+    assert state2.a == np.datetime64(100, 'D')
+
+
+def test_nan_inf_minmax():
+
+    data = Data(x=[3, 1, -2, np.inf, np.nan], label='test_data')
+
+    class SimpleState(State):
+
+        layer = CallbackProperty()
+        comp = CallbackProperty()
+        lower = CallbackProperty()
+        upper = CallbackProperty()
+        percentile = CallbackProperty()
+        log = CallbackProperty()
+
+    state = SimpleState()
+
+    helper = StateAttributeLimitsHelper(state, attribute='comp',  # noqa
+                                        lower='lower', upper='upper',
+                                        percentile='percentile', log='log')
+
+    state.data = data
+    state.comp = data.id['x']
+
+    assert state.lower == -2
+    assert state.upper == +3
+
+    state.log = True
+
+    assert state.lower == +1
+    assert state.upper == +3
+
+    state.log = False
+    state.percentile = 99
+
+    assert_allclose(state.lower, -1.97)
+    assert_allclose(state.upper, +2.98)
+
+
+def test_percentile_no_log():
+
+    # Regression test for a bug that caused a crash if the state class had a
+    # percentile attribute but no log.
+
+    data = Data(x=np.linspace(-100, 100, 10000),
+                y=np.linspace(2, 3, 10000), label='test_data')
+
+    class SimpleState(State):
+
+        layer = CallbackProperty()
+        comp = CallbackProperty()
+        lower = CallbackProperty()
+        upper = CallbackProperty()
+        scale = CallbackProperty()
+
+    state = SimpleState()
+
+    state.comp = data.id['x']
+    state.lower = 2
+    state.upper = 4
+
+    helper = StateAttributeLimitsHelper(state, attribute='comp',
+                                        lower='lower', upper='upper',
+                                        percentile='scale')
+
+    state.scale = 90
