@@ -188,6 +188,16 @@ class CallbackProperty(object):
         else:
             raise ValueError("Callback function not found: %s" % func)
 
+    def clear_callbacks(self, instance):
+        """
+        Remove all callbacks on this property.
+        """
+        for cb in [self._callbacks, self._2arg_callbacks]:
+            if instance in cb:
+                cb[instance].clear()
+        if instance in self._disabled:
+            self._disabled.pop(instance)
+
 
 class HasCallbackProperties(object):
     """
@@ -343,6 +353,14 @@ class HasCallbackProperties(object):
             if self.is_callback_property(name):
                 yield name, getattr(type(self), name)
 
+    def clear_callbacks(self):
+        """
+        Remove all global and property-specific callbacks.
+        """
+        self._global_callbacks.clear()
+        for name, prop in self.iter_callback_properties():
+            prop.clear_callbacks(self)
+
 
 def add_callback(instance, prop, callback, echo_old=False, priority=0):
     """
