@@ -86,9 +86,9 @@ from qtpy.QtCore import Qt
 from glue.core.layer_artist import MatplotlibLayerArtist
 from glue.config import qt_client
 from glue.core import Data
-from glue.core.edit_subset_mode import EditSubsetMode
 from glue.utils import nonpartial, as_list, all_artists, new_artists, remove_artists
 from glue import core
+from glue.external.echo.qt.connect import UserDataWrapper
 
 from glue.viewers.common.viz_client import GenericMplClient
 
@@ -96,7 +96,6 @@ from glue.viewers.matplotlib.qt.widget import MplWidget
 from glue.viewers.common.qt.data_viewer import DataViewer
 from glue.utils.qt.widget_properties import (ValueProperty, ButtonProperty,
                                              CurrentComboProperty)
-from glue.viewers.matplotlib.qt.toolbar import MatplotlibViewerToolbar
 
 __all__ = ["AttributeInfo", "ViewerState", "UserDefinedFunction",
            "CustomViewer", "SettingsOracleInterface", "SettingsOracle",
@@ -838,6 +837,7 @@ class CustomClient(GenericMplClient):
     def __init__(self, *args, **kwargs):
         self._coordinator = kwargs.pop('coordinator')
         kwargs.setdefault('axes_factory', self._coordinator.create_axes)
+        self.session = kwargs.pop('session', None)
         super(CustomClient, self).__init__(*args, **kwargs)
 
         self._coordinator.axes = self.axes
@@ -856,7 +856,7 @@ class CustomClient(GenericMplClient):
 
         s = self._coordinator._build_subset_state(roi=roi)
         if s:
-            EditSubsetMode().update(self.collect, s, focus_data=focus)
+            self.session.edit_subset_mode.update(self.collect, s, focus_data=focus)
 
     def _update_layer(self, layer):
         for artist in self.artists[layer]:
@@ -873,7 +873,7 @@ class CustomWidgetBase(DataViewer):
     LABEL = ''
 
     coordinator_cls = None
-    _toolbar_cls = MatplotlibViewerToolbar
+    tools = ['select:rectangle', 'select:polygon']
 
     def __init__(self, session, parent=None):
         super(CustomWidgetBase, self).__init__(session, parent)
@@ -885,7 +885,8 @@ class CustomWidgetBase(DataViewer):
         self.client = CustomClient(self._data,
                                    self.central_widget.canvas.fig,
                                    layer_artist_container=self._layer_artist_container,
-                                   coordinator=self._coordinator)
+                                   coordinator=self._coordinator,
+                                   session=session)
 
         self.statusBar().setSizeGripEnabled(False)
         self._update_artists = []
@@ -910,13 +911,6 @@ class CustomWidgetBase(DataViewer):
         self.client._redraw()
         self._coordinator.settings_changed()
 
-    @property
-    def tools(self):
-        if self._coordinator.selections_enabled:
-            return ['select:rectangle', 'select:polygon']
-        else:
-            return []
-
     def add_data(self, data):
         """Add a new data set to the widget
 
@@ -1377,13 +1371,12 @@ class ComponenentElement(FormElement, core.hub.HubListener):
         """
         Determine which components to list.
 
-
         This can be overridden by subclassing to limit which components are
         visible to the user.
 
         """
         comps = list(set([c for l in self.container.layers
-                          for c in l.data.components if not c._hidden]))
+                          for c in (l.data.main_components + l.data.derived_components)]))
         comps = sorted(comps, key=lambda x: x.label)
         return comps
 
@@ -1396,7 +1389,7 @@ class ComponenentElement(FormElement, core.hub.HubListener):
 
         comps = self._list_components()
         for c in comps:
-            combo.addItem(c.label, userData=c)
+            combo.addItem(c.label, userData=UserDataWrapper(c))
 
         try:
             combo.setCurrentIndex(comps.index(old))
@@ -1446,4 +1439,7 @@ class ChoiceElement(FormElement):
         return w
 
     def value(self, layer=None, view=None):
-        return self.params[self.ui.currentText()]
+        if self.ui.currentIndex() < 0:
+            return None
+        else:
+            return self.params[self.ui.currentText()]
